/* =======================================
 * JFreeChart : a Java Chart Class Library
 * =======================================
 *
 * Project Info:  http://www.jrefinery.com/jfreechart;
 * Project Lead:  David Gilbert (david.gilbert@jrefinery.com);
 *
 * (C) Copyright 2000-2002, by Simba Management Limited and Contributors.
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * -----------
 * XYPlot.java
 * -----------
 * (C) Copyright 2000-2002, by Simba Management Limited and Contributors.
 *
 * Original Author:  David Gilbert (for Simba Management Limited);
 * Contributor(s):   Craig MacFarlane;
 *                   Mark Watson (www.markwatson.com);
 *                   Jonathan Nash;
 *
 * $Id: XYPlot.java,v 1.11 2001/12/13 09:29:41 mungady Exp $
 *
 * Changes (from 21-Jun-2001)
 * --------------------------
 * 21-Jun-2001 : Removed redundant JFreeChart parameter from constructors (DG);
 * 18-Sep-2001 : Updated e-mail address and fixed DOS encoding problem (DG);
 * 15-Oct-2001 : Data source classes moved to com.jrefinery.data.* (DG);
 * 19-Oct-2001 : Removed the code for drawing the visual representation of each data point into
 *               a separate class StandardXYItemRenderer.  This will make it easier to add
 *               variations to the way the charts are drawn.  Based on code contributed by
 *               Mark Watson (DG);
 * 22-Oct-2001 : Renamed DataSource.java --> Dataset.java etc. (DG);
 * 20-Nov-2001 : Fixed clipping bug that shows up when chart is displayed inside JScrollPane (DG);
 * 12-Dec-2001 : Removed unnecessary 'throws' clauses from constructor (DG);
 * 13-Dec-2001 : Added skeleton code for tooltips.  Added new constructor. (DG);
 * 16-Jan-2002 : Renamed the tooltips class (DG);
 * 22-Jan-2002 : Added DrawInfo class, incorporating tooltips and crosshairs.  Crosshairs based
 *               on code by Jonathan Nash (DG);
 *
 */

package com.jrefinery.chart;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import com.jrefinery.chart.event.*;
import com.jrefinery.chart.tooltips.*;
import com.jrefinery.data.*;

/**
 * A general class for plotting data in the form of (x, y) pairs.  XYPlot can use data from any
 * class that implements the XYDataset interface (in the com.jrefinery.data package).
 * <P>
 * XYPlot makes use of a renderer to draw each point on the plot.  By using different renderers,
 * various chart types can be produced.  The ChartFactory class contains static methods for
 * creating pre-configured charts.
 * @see ChartFactory
 * @see Plot
 * @see XYDataset
 */
public class XYPlot extends Plot implements HorizontalValuePlot, VerticalValuePlot {

    /** Object responsible for drawing the visual representation of each point on the plot. */
    protected XYItemRenderer renderer;

    /** A list of (optional) vertical lines that will be overlaid on the plot. */
    protected ArrayList verticalLines = null;

    /** The colors for the vertical lines. */
    protected ArrayList verticalColors = null;

    /** A list of horizontal lines that will be overlaid on the plot. */
    protected ArrayList horizontalLines = null;

    /** The colors for the horizontal lines. */
    protected ArrayList horizontalColors = null;

    /** The tool tip generator. */
    protected XYToolTipGenerator toolTipGenerator;

    /**
     * Constructs an XYPlot with the specified axes (other attributes take default values).
     * @param horizontalAxis The horizontal axis.
     * @param verticalAxis The vertical axis.
     */
    public XYPlot(ValueAxis horizontalAxis, ValueAxis verticalAxis) {

	this(horizontalAxis, verticalAxis,
             Plot.DEFAULT_INSETS,
             Plot.DEFAULT_BACKGROUND_COLOR,
             Plot.DEFAULT_OUTLINE_STROKE,
             Plot.DEFAULT_OUTLINE_COLOR);

    }

    /**
     * Constructs a new XY plot.
     * @param horizontalAxis The horizontal axis.
     * @param verticalAxis The vertical axis.
     * @param insets Amount of blank space around the plot area.
     * @param background The Paint used to fill the plot background.
     * @param outlineStroke The Stroke used to draw an outline around the plot.
     * @param outlinePaint The color used to draw the plot outline.
     */
    public XYPlot(ValueAxis horizontalAxis, ValueAxis verticalAxis,
                  Insets insets, Paint background,
                  Stroke outlineStroke, Paint outlinePaint) {

        super(horizontalAxis, verticalAxis, insets, background, outlineStroke, outlinePaint);
        this.renderer = new StandardXYItemRenderer();
        this.toolTipGenerator = null;

    }

    /**
     * Returns a reference to the current item renderer.
     * @return A reference to the current item renderer.
     */
    public XYItemRenderer getItemRenderer() {
        return this.renderer;
    }

    /**
     * Sets the item renderer, and notifies all listeners of a change to the plot.
     * @param renderer The new renderer.
     */
    public void setXYItemRenderer(XYItemRenderer renderer) {
        this.renderer = renderer;
        this.notifyListeners(new PlotChangeEvent(this));
    }

    /**
     * A convenience method that returns the dataset for the plot, cast as an XYDataset.
     * @return The dataset for the plot, cast as an XYDataset.
     */
    public XYDataset getDataset() {
	return (XYDataset)chart.getDataset();
    }

    /**
     * Adds a vertical line at location with default color blue.
     * @return void
     */
    public void addVerticalLine(Number location) {
        addVerticalLine(location, Color.blue);
    }

    /**
     * Adds a vertical of the given color at location with the given color.
     * @return void
     */
    public void addVerticalLine(Number location, Paint color) {

        if (verticalLines == null) {
            verticalLines = new ArrayList();
            verticalColors = new ArrayList();
        }

        verticalColors.add(color);
        verticalLines.add(location);

    }

    /**
     * Adds a horizontal line at the specified data value, using the default color red.
     * @param value The data value.
     */
    public void addHorizontalLine(Number value) {

        addHorizontalLine(value, Color.red);
        this.notifyListeners(new PlotChangeEvent(this));
    }

    /**
     * Adds a horizontal line at the specified data value, using the specified color.
     * @param value The data value.
     * @param color The line color.
     */
    public void addHorizontalLine(Number location, Paint color) {

        if (horizontalLines == null) {
            horizontalLines = new ArrayList();
            horizontalColors = new ArrayList();
        }

        horizontalColors.add(color);
        horizontalLines.add(location);

    }

    /**
     * A convenience method that returns a reference to the horizontal axis cast as a
     * ValueAxis.
     * @return The horizontal axis cast as a ValueAxis.
     */
    public ValueAxis getDomainAxis() {
	return (ValueAxis)horizontalAxis;
    }

    /**
     * A convenience method that returns a reference to the vertical axis cast as a
     * ValueAxis.
     * @return The vertical axis cast as a ValueAxis.
     */
    public ValueAxis getRangeAxis() {
	return (ValueAxis)verticalAxis;
    }

    /**
     * Checks the compatibility of a horizontal axis, returning true if the axis is compatible with
     * the plot, and false otherwise.
     * @param axis The horizontal axis;
     * @return True if the axis is compatible with the plot, and false otherwise.
     */
    public boolean isCompatibleHorizontalAxis(Axis axis) {
	if (axis instanceof HorizontalNumberAxis) {
	    return true;
	}
	else if (axis instanceof HorizontalDateAxis) {
	    return true;
	}
	else return false;
    }

    /**
     * Checks the compatibility of a vertical axis, returning true if the axis is compatible with
     * the plot, and false otherwise.
     * @param axis The vertical axis;
     * @return True if the axis is compatible with the plot, and false otherwise.
     */
    public boolean isCompatibleVerticalAxis(Axis axis) {
	if (axis instanceof VerticalNumberAxis) {
	    return true;
	}
	else return false;
    }

    /**
     * Draws the XY plot on a Java 2D graphics device (such as the screen or a printer).
     * <P>
     * XYPlot now relies on an XYItemRenderer to draw each item in the plot.  This allows the
     * visual representation of the plot to be changed easily.
     * @param g2 The graphics device.
     * @param drawArea The area within which the plot (including axis labels) should be drawn.
     * @param info Collects chart drawing information (null permitted).
     */
    public void draw(Graphics2D g2, Rectangle2D plotArea, DrawInfo info) {

        // set up info collection...
        ToolTipsCollection tooltips = null;
        if (info!=null) {
            info.setPlotArea(plotArea);
            tooltips = info.getToolTipsCollection();
        }

        // adjust the drawing area for plot insets (if any)...
	if (insets!=null) {
	    plotArea.setRect(plotArea.getX()+insets.left,
                             plotArea.getY()+insets.top,
                             plotArea.getWidth()-insets.left-insets.right,
                             plotArea.getHeight()-insets.top-insets.bottom);
	}

	// estimate the area required for drawing the axes...
	HorizontalAxis hAxis = getHorizontalAxis();
	VerticalAxis vAxis = getVerticalAxis();
	double hAxisAreaHeight = hAxis.reserveHeight(g2, this, plotArea);
	Rectangle2D vAxisArea = vAxis.reserveAxisArea(g2, this, plotArea, hAxisAreaHeight);

        // ...and therefore what is left for the plot itself...
	Rectangle2D dataArea = new Rectangle2D.Double(plotArea.getX()+vAxisArea.getWidth(),
                                                      plotArea.getY(),
                                                      plotArea.getWidth()-vAxisArea.getWidth(),
                                                      plotArea.getHeight()-hAxisAreaHeight);

        if (info!=null) {
            info.setDataArea(dataArea);
        }

        CrosshairInfo crosshairInfo = new CrosshairInfo();

        crosshairInfo.setCrosshairDistance(Double.POSITIVE_INFINITY);
        crosshairInfo.setAnchorX(this.getDomainAxis().getAnchorValue());
        crosshairInfo.setAnchorY(this.getRangeAxis().getAnchorValue());

        // draw the plot background and axes...
	drawOutlineAndBackground(g2, dataArea);
	this.horizontalAxis.draw(g2, plotArea, dataArea);
	this.verticalAxis.draw(g2, plotArea, dataArea);

        // now get the data and plot it (the visual representation will depend on the renderer
        // that has been set)...
        XYDataset data = this.getDataset();
        if (data!=null) {
	    Shape originalClip = g2.getClip();
	    g2.clip(dataArea);

            drawVerticalLines(g2, dataArea);
            drawHorizontalLines(g2, dataArea);

            double transRangeZero = this.getRangeAxis().translateValueToJava2D(0.0, dataArea);

            int seriesCount = data.getSeriesCount();
            for (int series=0; series<seriesCount; series++) {
                int itemCount = data.getItemCount(series);
                for (int item=0; item<itemCount; item++) {
                    Shape tooltipArea = renderer.drawItem(g2, dataArea, info, this,
                                                          (ValueAxis)horizontalAxis,
                                                          (ValueAxis)verticalAxis,
                                                          data, series, item,
                                                          transRangeZero, crosshairInfo);

                    // add a tooltip for the item...
                    if (tooltips!=null) {
                        if (this.toolTipGenerator==null) {
                            toolTipGenerator = new StandardXYToolTipGenerator();
                        }
                        String tip = this.toolTipGenerator.generateToolTip(data, series, item);
                        if (tooltipArea!=null) {
                            tooltips.addToolTip(tip, tooltipArea);
                        }
                    }

                }
            }


            // draw vertical crosshair if required...
            ValueAxis hva = (ValueAxis)this.horizontalAxis;
            hva.setCrosshairValue(crosshairInfo.getCrosshairX());
            if (hva.isCrosshairVisible()) {
                this.drawVerticalLine(g2, dataArea, hva.getCrosshairValue(),
                                      hva.getCrosshairStroke(),
                                      hva.getCrosshairPaint());
            }

            // draw horizontal crosshair if required...
            ValueAxis vva = (ValueAxis)this.verticalAxis;
            vva.setCrosshairValue(crosshairInfo.getCrosshairY());
            if (vva.isCrosshairVisible()) {
                this.drawHorizontalLine(g2, dataArea, vva.getCrosshairValue(),
                                        vva.getCrosshairStroke(),
                                        vva.getCrosshairPaint());
            }

            g2.setClip(originalClip);
        }
    }

    /**
     * Utility method for drawing a crosshair on the chart (if required).
     */
    private void drawVerticalLine(Graphics2D g2, Rectangle2D dataArea, double value,
                                  Stroke stroke, Paint paint) {

        double xx = this.getDomainAxis().translateValueToJava2D(value, dataArea);
        Line2D line = new Line2D.Double(xx, dataArea.getMinY(), xx, dataArea.getMaxY());
        g2.setStroke(stroke);
        g2.setPaint(paint);
        g2.draw(line);

    }

    /**
     * Utility method for drawing a crosshair on the chart (if required).
     */
    private void drawHorizontalLine(Graphics2D g2, Rectangle2D dataArea, double value,
                                    Stroke stroke, Paint paint) {

        double yy = this.getRangeAxis().translateValueToJava2D(value, dataArea);
        Line2D line = new Line2D.Double(dataArea.getMinX(), yy, dataArea.getMaxX(), yy);
        g2.setStroke(stroke);
        g2.setPaint(paint);
        g2.draw(line);

    }

    /**
     * Support method for the draw(...) method.
     */
    private void drawVerticalLines(Graphics2D g2, Rectangle2D dataArea) {

        // Draw any vertical lines
        if (verticalLines != null) {
            for (int i=0; i<verticalLines.size(); i++) {
                g2.setPaint((Paint)verticalColors.get(i));
                g2.setStroke(new BasicStroke(1));
                Number x = (Number)verticalLines.get(i);
                int xint = (int)getDomainAxis().translateValueToJava2D(x.doubleValue(), dataArea);
                g2.drawLine(xint, 0, xint, (int)(dataArea.getHeight()));
            }
        }

    }

    /**
     * Support method for the draw(...) method.
     */
    private void drawHorizontalLines(Graphics2D g2, Rectangle2D dataArea) {

        // Draw any horizontal lines
        if (horizontalLines != null) {
            for (int i=0; i<horizontalLines.size(); i++) {
                g2.setPaint((Paint)horizontalColors.get(i));
                g2.setStroke(new BasicStroke(1));
                Number y = (Number)horizontalLines.get(i);
                int yint = (int)getRangeAxis().translateValueToJava2D(y.doubleValue(), dataArea);
                g2.drawLine(0, yint, (int)(dataArea.getWidth()), yint);
            }
        }

    }

    /**
     * Handles a 'click' on the plot by updating the anchor values...
     */
    public void handleClick(int x, int y, DrawInfo info) {

        // set the anchor value for the horizontal axis...
        ValueAxis hva = this.getDomainAxis();
        double hvalue = hva.translateJava2DtoValue((float)x, info.getDataArea());

        hva.setAnchorValue(hvalue);
        hva.setCrosshairValue(hvalue);

        // set the anchor value for the vertical axis...
        ValueAxis vva = this.getRangeAxis();
        double vvalue = vva.translateJava2DtoValue((float)y, info.getDataArea());
        vva.setAnchorValue(vvalue);
        vva.setCrosshairValue(vvalue);

    }

    public void zoom(double percent) {

        if (percent>0) {
            ValueAxis domainAxis = this.getDomainAxis();
            double range = domainAxis.getMaximumAxisValue()-domainAxis.getMinimumAxisValue();
            double scaledRange = range * percent;
            domainAxis.setAnchoredRange(scaledRange);

            ValueAxis rangeAxis = this.getRangeAxis();
            range = rangeAxis.getMaximumAxisValue()-rangeAxis.getMinimumAxisValue();
            scaledRange = range * percent;
            rangeAxis.setAnchoredRange(scaledRange);
        }
        else {
            this.getRangeAxis().setAutoRange(true);
            this.getDomainAxis().setAutoRange(true);
        }

    }

    /**
     * Returns the plot type as a string.
     * @return A short string describing the type of plot.
     */
    public String getPlotType() {
	return "XY Plot";
    }

    /**
     * Returns the minimum value in the domain, since this is plotted against the horizontal axis
     * for an XYPlot.
     * @return The minimum value to be plotted against the horizontal axis.
     */
    public Number getMinimumHorizontalDataValue() {

	Dataset data = this.getChart().getDataset();
	if (data!=null) {
	    return Datasets.getMinimumDomainValue(data);
	}
	else return null;

    }

    /**
     * Returns the maximum value in the domain, since this is plotted against the horizontal axis
     * for an XYPlot.
     * @return The maximum value to be plotted against the horizontal axis.
     */
    public Number getMaximumHorizontalDataValue() {

	Dataset data = this.getChart().getDataset();
	if (data!=null) {
	    return Datasets.getMaximumDomainValue(data);
	}
	else return null;

    }

    /**
     * Returns the minimum value in the range, since this is plotted against the vertical axis for
     * an XYPlot.
     * @return The minimum value to be plotted against the vertical axis.
     */
    public Number getMinimumVerticalDataValue() {

	Dataset data = this.getChart().getDataset();
	if (data!=null) {
	    return Datasets.getMinimumRangeValue(data);
	}
	else return null;

    }

    /**
     * Returns the maximum value in the range, since this is plotted against the vertical axis for
     * an XYPlot.
     * @return The maximum value to be plotted against the vertical axis.
     */
    public Number getMaximumVerticalDataValue() {

	Dataset data = this.getChart().getDataset();
	if (data!=null) {
	    return Datasets.getMaximumRangeValue(data);
	}
	else return null;
    }

}